@@ -25,7 +25,7 @@ django.setup()
 
 from preservationdatabase.models import CarinianaPreservation, \
     ClockssPreservation, HathiPreservation, LockssPreservation, \
-    PKPPreservation, PorticoPreservation
+    OculScholarsPortalPreservation, PKPPreservation, PorticoPreservation
 from django.db import transaction
 import utils
 
@@ -105,6 +105,24 @@ def import_hathi(file, bucket):
         file, bucket=bucket, s3client=s3client)
 
 
+@click.command()
+@click.option('--file',
+              default='scholars_portal_keepers_20230202.xml',
+              help='The filename of the OCUL full dump to use')
+@click.option('--bucket',
+              default='preservation.research.crossref.org',
+              help='The s3 bucket from which to retrieve the data')
+@transaction.atomic()
+def import_ocul(file, bucket):
+    """Import data from Ocul (requires local file download or S3)"""
+
+    s3client = boto3.client('s3')
+    OculScholarsPortalPreservation.import_data(file, bucket=bucket,
+                                               s3client=s3client)
+
+    return
+
+
 @click.command()
 def import_all():
     """Download and import all data (excluding HathiTrust)"""
@@ -158,6 +176,7 @@ if __name__ == '__main__':
     cli.add_command(import_clockss)
     cli.add_command(import_hathi)
     cli.add_command(import_lockss)
+    cli.add_command(import_ocul)
     cli.add_command(import_pkp)
     cli.add_command(import_portico)
     cli.add_command(show_preservation)
