@@ -645,5 +645,5 @@ class OculScholarsPortalPreservation(models.Model):
                 publisher=publisher
             )
 
-            logging.info(f'Added {output["title"]} ({volume}.{issue}) to '
-                         f'{OculScholarsPortalPreservation.name()} data')
+        logging.info(f'Added {output["title"]} to '
+                     f'{OculScholarsPortalPreservation.name()} data')
