@@ -6,7 +6,6 @@ from io import StringIO
 import requests
 from crossref.restful import Etiquette, Works
 from django.db.models import QuerySet
-
 from lxml import etree as ET
 
 
@@ -154,14 +153,20 @@ def clear_out(model):
     model.objects.all().delete()
 
 
-def download_remote(local, model, url, bucket="", s3client=None, decode=True):
+def download_remote(local, model, url, bucket="", s3client=None, decode=True,
+                    file=False, filename=''):
     if s3client:
-        if decode:
-            return s3client.get_object(Bucket=bucket, Key=url)[
-                'Body'].read().decode('utf-8')
+        if not file:
+            if decode:
+                return s3client.get_object(Bucket=bucket, Key=url)[
+                    'Body'].read().decode('utf-8')
+            else:
+                return s3client.get_object(Bucket=bucket, Key=url)[
+                    'Body'].read()
         else:
-            return s3client.get_object(Bucket=bucket, Key=url)[
-                'Body'].read()
+            logging.info('Storing downloaded file as {}'.format(filename))
+            s3client.download_file(bucket, url, filename)
+            return filename
 
     if not local:
         logging.info(f'Downloading: {model.name()} data')
@@ -252,76 +257,102 @@ def get_preserved_item_record(model, container_title, issn) -> QuerySet | None:
     return preserved_item
 
 
-def process_onix(xml_file, elements, callback):
+def process_onix(xml_file, elements, callback) -> None:
     """
     A faster method for processing ONIX XML than using DOM methods
     """
-    root = ET.XML(xml_file)
+    current_object = {'volumes': [], 'issues': []}
 
-    outputs = []
+    collect = False
 
-    for holdings_record in root.findall(".//{http://www.editeur.org/onix/serials/SOH}HoldingsRecord"):
+    for event, elem in ET.iterparse(xml_file, events=('start', 'end')):
 
-        resource_versions = holdings_record.findall(".//{http://www.editeur.org/onix/serials/SOH}ResourceVersion")
-        for resource_version in resource_versions:
+        if event == 'end' and elem.tag == '{http://www.editeur.org/onix/' \
+                                          'serials/SOH}HoldingsRecord':
+            callback(current_object)
 
-            resource_version_identifiers = resource_version.findall(".//{http://www.editeur.org/onix/serials/SOH}ResourceVersionIdentifier")
-            for resource_version_identifier in resource_version_identifiers:
+            current_object = {'volumes': [], 'issues': []}
 
-                resource_type = resource_version_identifier.find(".//{http://www.editeur.org/onix/serials/SOH}ResourceVersionIDType")
+        # detect if it's an ISSN
+        if event == 'start' \
+                and elem.tag == '{http://www.editeur.org/onix/' \
+                                'serials/SOH}ResourceVersionIDType' \
+                and elem.text == '07':
+            collect = True
+        elif event == 'start' \
+                and elem.tag == '{http://www.editeur.org/onix/' \
+                                'serials/SOH}ResourceVersionIDType' \
+                and elem.text != '07':
+            collect = False
 
-                # determine if ISSN
-                if int(resource_type.text) == 7:
-                    # extract unhyphenated ISSN, title, and publisher
-                    issn = resource_version_identifier.find(".//{http://www.editeur.org/onix/serials/SOH}IDValue").text
-                    title = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}TitleText").text
-                    publisher = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}PublisherName").text
+        if event == 'start' and elem.tag == '{http://www.editeur.org/onix/' \
+                                            'serials/SOH}IDValue' and collect:
+            current_object['issn'] = normalize_issn(elem.text)
 
-                    volume = None
-                    issue = None
+        if event == 'start' and elem.tag == '{http://www.editeur.org/onix/' \
+                                            'serials/SOH}TitleText' and collect:
+            current_object['title'] = elem.text
 
-                    # extract volume and issue
-                    coverages = resource_version.findall(".//{http://www.editeur.org/onix/serials/SOH}Coverage")
+        if event == 'start' \
+                and elem.tag == '{http://www.editeur.org/onix/' \
+                                'serials/SOH}PublisherName' \
+                and collect:
+            current_object['publisher'] = elem.text
 
-                    for coverage in coverages:
-                        level_one = coverage.find(".//{http://www.editeur.org/onix/serials/SOH}Level1")
-                        level_two = coverage.find(".//{http://www.editeur.org/onix/serials/SOH}Level2")
+        if event == 'start' \
+                and elem.tag == '{http://www.editeur.org/onix/' \
+                                'serials/SOH}Coverage' and collect:
+            coverage = elem
 
-                        levels = [level_one, level_two]
+            volume = ''
+            issue = ''
 
-                        for level in levels:
-                            unit = level.find(".//{http://www.editeur.org/onix/serials/SOH}Unit").text
-                            number = level.find(".//{http://www.editeur.org/onix/serials/SOH}Number").text
+            level_one = coverage.find(
+                ".//{http://www.editeur.org/onix/serials/SOH}Level1")
+            level_two = coverage.find(
+                ".//{http://www.editeur.org/onix/serials/SOH}Level2")
 
-                            if unit == 'Volume':
-                                volume = number
-                            elif unit == 'Issue':
-                                issue = number
-                            else:
-                                print('Unrecognised unit: {}'.format(unit))
+            levels = [level_one, level_two]
+            unit = None
 
-                    # preservation and verification status
-                    try:
-                        status = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}PreservationStatusCode").text
-                        verification = resource_version.find(".//{http://www.editeur.org/onix/serials/SOH}VerificationStatus").text
+            for level in levels:
+                if level:
+                    unit = level.find(
+                        ".//{http://www.editeur.org/onix/serials/SOH}Unit")
 
-                        output = {'issn': normalize_issn(issn), 'title': title,
-                                  'publisher': publisher, 'volume': volume,
-                                  'issue': issue, 'status': status,
-                                  'verification': verification}
+                    # no idea why, but this HAS to be an explicit comparison
+                    # with None rather than a boolean comparison
+                    unit = unit.text if unit is not None else None
 
-                        outputs.append(output)
+                    number = level.find(
+                        ".//{http://www.editeur.org/onix/serials/SOH}Number")
 
-                        if callback:
-                            callback(output)
+                    number = number.text if number is not None else None
 
-                    except AttributeError:
-                        # if we get here, there is no preservation or
-                        # verification status
-                        # print("ERROR on {}".format(issn))
-                        pass
+                    if unit == 'Volume':
+                        volume = number
+                    elif unit == 'Issue':
+                        issue = number
+                    else:
+                        print('Unrecognised unit: {}'.format(unit))
 
-    return outputs
+            if unit:
+                current_object['volumes'].append(volume)
+                current_object['issues'].append(issue)
+
+        if event == 'start' \
+                and elem.tag == '{http://www.editeur.org/onix/' \
+                                'serials/SOH}PreservationStatusCode'\
+                and collect:
+            current_object['status'] = elem.text
+
+        if event == 'start' \
+                and elem.tag == '{http://www.editeur.org/onix/' \
+                                'serials/SOH}VerificationStatus'\
+                and collect:
+            current_object['verification'] = elem.text
+
+    return
 
 
 def normalize_issn(issn) -> str:
